@@ -1,5 +1,8 @@
-;; Incremental search for SKK (version 1.9 of April 21, 1991)
+;; Incremental search for SKK (version 1.12 of October 14, 1992)
 
+;; version 1.12 released on October 14, 1992
+;; version 1.11 released on April 13, 1992
+;; version 1.10 released on March 26, 1992
 ;; version 1.9 released on April 21, 1991
 ;; version 1.8 released on April 18, 1991
 ;; version 1.7 released on April 17, 1991
@@ -8,8 +11,11 @@
 ;; version 1.4 released on October 1, 1990
 ;; version 1.3 released on March 22, 1990
 ;; version 1.2 released on April 10, 1989
+
 ;; Masahiko Sato modified the original incremental search.
 ;; Tsugutomo Enami (enami@ptgd.sony.co.jp) contributed modifications.
+;; Masakazu Takahashi (masaka-t@ascii.co.jp) contributed modifications.
+;; Koichi Eryou (k_era@pluto.ai.kyutech.ac.jp) contributed modifications.
 
 ;; Incremental search
 ;; Copyright (C) 1985, 1986 Free Software Foundation, Inc.
@@ -65,6 +71,26 @@
 ;This is the style where a one-line window is created to show the line
 ;that the search has reached.")
 
+;; suggested by Tsugutomo Enami, 1991.5.27.
+(require 'skk)
+
+(defvar skk-isearch-use-previous-mode nil
+  "*If non-nil, search mode will be same as the last search mode for the
+previous search in the buffer.")
+(make-variable-buffer-local 'skk-isearch-use-previous-mode)
+
+;; suggested by Yoshiyuki Kondo, 1991.6.19.
+(defvar skk-isearch-start-mode nil
+  "*Specifies the search mode when isearch is called.  nil -> if skk-mode
+has been called in this buffer, same as the mode of the buffer, otherwise
+ascii search. ascii -> ascii search.  kana -> hira kana search.")
+(make-variable-buffer-local 'skk-isearch-start-mode)
+
+(defvar skk-isearch-mode nil
+  "Current search mode.  0 -> hira kana search.  1 -> kata kana search.
+2 -> zenei search.  3 -> ascii search.")
+(make-variable-buffer-local 'skk-isearch-mode)
+
 (defvar j-ignore-exp "[\n\t <>|]*"
   "*If non-nil, it is inserted between consecutive JIS characters in the
 search string.")
@@ -93,82 +119,38 @@ the keyboard if CHAR does not correspond to a command."
 (defun skk-isearch-forward (&optional prefix)
   "Forward incremental search under SKK"
   (interactive "P")
-  ;; if SKK has not been invoked yet in this Emacs session,
-  ;; call it now, and initialize some variables.
-  (or (featurep 'skk)
-      (save-excursion
-	(set-buffer (get-buffer-create " *search-buffer* "))
-	(skk-mode 1)))
   (let ((j-ignore-exp (if prefix nil j-ignore-exp))
-	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp))
-	(j-mode (if skk-mode
-		    ;; if SKK is active on this buffer, then use its j-mode
-		    j-mode
-		  ;; otherwise, check if the buffer contains kanji code
-		  (if (check-region-kanji-code (point-min) (point-max))
-		      t
-		    0))))
-    (if (eq j-mode 0) (isearch t) (skk-isearch t))))
+	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp)))
+    (if (check-region-kanji-code (point-min) (point-max))
+	(skk-isearch t)
+      (isearch t))))
 
 (defun skk-isearch-backward (&optional prefix)
   "Backward incremental search under SKK"
   (interactive "P")
-  ;; if SKK has not been invoked yet in this Emacs session,
-  ;; call it now, and initialize some variables.
-  (or (featurep 'skk)
-      (save-excursion
-	(set-buffer (get-buffer-create " *search-buffer* "))
-	(skk-mode 1)))
   (let ((j-ignore-exp (if prefix nil j-ignore-exp))
-	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp))
-	(j-mode (if skk-mode
-		    ;; if SKK is active on this buffer, then use its j-mode
-		    j-mode
-		  ;; otherwise, check if the buffer contains kanji code
-		  (if (check-region-kanji-code (point-min) (point-max))
-		      t
-		    0))))
-    (if (eq j-mode 0) (isearch nil) (skk-isearch nil))))
+	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp)))
+    (if (check-region-kanji-code (point-min) (point-max))
+	(skk-isearch nil)
+      (isearch nil))))
 
 (defun skk-isearch-forward-regexp (&optional prefix)
   "Forward incremental search for regular expression under SKK"
   (interactive "P")
-  ;; if SKK has not been invoked yet in this Emacs session,
-  ;; call it now, and initialize some variables.
-  (or (featurep 'skk)
-      (save-excursion
-	(set-buffer (get-buffer-create " *search-buffer* "))
-	(skk-mode 1)))
   (let ((j-ignore-exp (if prefix nil j-ignore-exp))
-	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp))
-	(j-mode (if skk-mode
-		    ;; if SKK is active on this buffer, then use its j-mode
-		    j-mode
-		  ;; otherwise, check if the buffer contains kanji code
-		  (if (check-region-kanji-code (point-min) (point-max))
-		      t
-		    0))))
-    (if (eq j-mode 0) (isearch t t) (skk-isearch t t))))
+	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp)))
+    (if (check-region-kanji-code (point-min) (point-max))
+	(skk-isearch t t)
+      (isearch t t))))
 
 (defun skk-isearch-backward-regexp (&optional prefix)
   "Backward incremental search for regular expression under SKK"
   (interactive "P")
-  ;; if SKK has not been invoked yet in this Emacs session,
-  ;; call it now, and initialize some variables.
-  (or (featurep 'skk)
-      (save-excursion
-	(set-buffer (get-buffer-create " *search-buffer* "))
-	(skk-mode 1)))
   (let ((j-ignore-exp (if prefix nil j-ignore-exp))
-	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp))
-	(j-mode (if skk-mode
-		    ;; if SKK is active on this buffer, then use its j-mode
-		    j-mode
-		  ;; otherwise, check if the buffer contains kanji code
-		  (if (check-region-kanji-code (point-min) (point-max))
-		      t
-		    0))))
-    (if (eq j-mode 0) (isearch nil t) (skk-isearch nil t))))
+	(j-ascii-ignore-exp (if prefix nil j-ascii-ignore-exp)))
+    (if (check-region-kanji-code (point-min) (point-max))
+	(skk-isearch nil t)
+      (isearch nil t))))
 
 ;; This function does all the work of incremental search under SKK.
 
@@ -179,6 +161,10 @@ the keyboard if CHAR does not correspond to a command."
 	(j j-mode)
 	(k j-katakana)
 	(z j-zenkaku)
+	(s skk-mode)
+	(iupm skk-isearch-use-previous-mode)
+	(im skk-isearch-mode)
+	(ism skk-isearch-start-mode)
 	(skk-echo nil) ; inhibit echoing of roman prefix
 	;(skk-isearch-message nil)
 	quote
@@ -207,10 +193,42 @@ the keyboard if CHAR does not correspond to a command."
       ;; modified by Tsugutomo Enami, 1990.7.13
       (skk-mode 1)
       (j-kakutei)
-      (if j
-	  (if k (j-toggle-kana))
-	(if z (j-zenkaku-eiji)
-	  (j-mode-off))))
+      (if (and iupm im)
+	  (cond ((eq im 0) ;; hira kana search
+		 nil)
+		((eq im 1) ;; kata kana search
+		 ;; modified at the suggestion of Koichi Eryou, 1992.10.13
+		 (setq j-katakana (not j-katakana)))
+		((eq im 2) ;; zenei serach
+		 (j-zenkaku-eiji))
+		((eq im 3) ;; ascii serach
+		 (j-mode-off))
+		(t (error "Invalid skk-isearch-mode!")))
+	(cond ((null ism)
+	       (if s
+		   (if j
+		       (if k
+			   (progn
+			     (setq im 1)
+			     ;; modified at the suggestion of Koichi Eryou,
+			     ;; 1992.10.13
+			     (setq j-katakana (not j-katakana)))
+			 (setq im 0))
+		     (if z
+			 (progn
+			   (setq im 2)
+			   (j-zenkaku-eiji))
+		       (setq im 3)
+		       (j-mode-off)))
+		 (setq im 3)
+		 (j-mode-off)))
+	      ((eq ism 'ascii)
+	       (setq im 3)
+	       (j-mode-off))
+	      ((eq ism 'kana)
+	       (setq im 0))
+	      (t (error "Invalid skk-isearch-start-mode!")))))
+    (setq skk-isearch-mode im)
     (j-isearch-push-state)
     (save-window-excursion
      (catch 'search-done
@@ -458,8 +476,15 @@ the keyboard if CHAR does not correspond to a command."
 		   (if c-q-hack "^Q" "")
 		   (if invalid-regexp
 		       (concat " [" invalid-regexp "]")
-		     ""))))
+		     "")))
+	sm)
     (aset m 0 (upcase (aref m 0)))
+    (save-excursion
+      (set-buffer search-buff)
+      (setq sm
+	    (if j-mode (if j-katakana 1 0)
+	      (if j-zenkaku 2 3))))
+    (setq skk-isearch-mode sm)
     ;(setq M search-message); for debug, masahiko
     (let ((cursor-in-echo-area ellipsis))
       (if c-q-hack m (message "%s" m)))))
@@ -600,3 +625,16 @@ the keyboard if CHAR does not correspond to a command."
     (setq command-history (cons (list function string) command-history))
     ;; Go ahead and search.
     (funcall function string)))
+
+;; patch for old Nemacs (v2)
+;; suggested by Masakazu Takahashi, 1992.4.12.
+;; this modification is necessary since the function
+;; check-region-kanji-code is unbound in Nemacs v2.
+
+(or (fboundp 'check-region-kanji-code)
+    ;; we define check-region-kanji-code as a function which simply
+    ;; returns t.  it doesn't work like the real check-region-kanji-code,
+    ;; but we don't care.
+    (defun check-region-kanji-code (p q) t))
+
+;; end of patch for old Nemacs (v2)
