@@ -1,14 +1,9 @@
 ;; skk-tools: A package of utility functions for SKK.
 
-;; version 1.11 of May 9, 1992
-;; version 1.10 of April 23, 1992
-;; version 1.9 of June 9, 1991
-;; version 1.8 of June 6, 1991
-;; version 1.7 of May 15, 1991
-;; version 1.6 of May 8, 1991
-;; version 1.5 of April 17, 1991
-;; version 1.4 of April 12, 1991
-;; Copyright (C) 1991, 1992 Masahiko Sato (masahiko@sato.riec.tohoku.ac.jp)
+;; Copyright (C) 1991, 1992, 1993
+;; Masahiko Sato (masahiko@sato.riec.tohoku.ac.jp)
+
+;; version 1.13 of April 24, 1993
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -29,8 +24,22 @@
 ;;	Brian Thomson (thomson@hub.toronto.edu)
 ;;      Tomoyuki Hiro (hiro@momo.it.okayama-u.ac.jp)
 
+;; version 1.13 released 1993.4.24
+;; version 1.12 released 1993.1.22
+;; version 1.11 released 1992.5.9
+;; version 1.10 released 1992.4.23
+;; version 1.9  released 1991.6.9
+;; version 1.8  released 1991.6.6
+;; version 1.7  released 1991.5.15
+;; version 1.6  released 1991.5.8
+;; version 1.5  released 1991.4.17
+;; version 1.4  released 1991.4.12
+
 (provide 'skk-tools)
 
+(defvar skk-emacs-sort (concat exec-directory "emacs-sort")
+  "Full path name of the emacs sort shell script.")
+
 ;; Part I
 ;; SKK diff: Compute the difference of two SKK dictionaries.
 ;; Modified by Masakazu Takahashi (masaka-t@ascii.co.jp), 1991.1.8., 1991.3.31
@@ -59,8 +68,8 @@
 ;;     Ƥ褤. ʾνɬפʤйԤ, ̤Ƥ dic1 ˥֤.
 ;; [6] M-x skk-diff ¹Ԥ. ּե1פȡּե2פ
 ;;      ʹƤΤ, 줾, dic1  dic2 . ׻λ,
-;;      ̤ SKK-JISYO-DIFF Ȥ̾Υե dic1  dic2 κʬ
-;;      񤫤.
+;;      ̤ SKK-JISYO-DIFF Ȥ̾Υե dic1  dic2 κʬ(dic1
+;;      ˤ뤬 dic2 ˤϤʤ)񤫤.
 
 (defun skk-diff (file1 file2)
   (interactive "fե1: \nfե2: ")
@@ -78,7 +87,7 @@
       (j-delete-okuri-entries)
       ;; sort the content of file1
       (message "sorting the first file...")
-      (sort-lines nil (point-min) (point-max))
+      (skk-sort-lines nil (point-min) (point-max))
       (message "sorting the first file...done")
       (goto-char (point-max))
       (setq lines (count-lines (point-min) (point-max)))
@@ -95,7 +104,7 @@
 	    (sit-for 3)))
       ;; sort the content of file2
       (message "sorting the second file...")
-      (sort-lines nil (+ q 0) (point-max))
+      (skk-sort-lines nil (+ q 0) (point-max))
       (message "sorting the second file...done")
       (goto-char (point-min))
       (setq p (point-marker))
@@ -319,12 +328,17 @@
   (interactive "fե#1: \nfե#2: ")
   (save-excursion
     (skk-merge-load file1 file2)
-    (sort-fields 1 (point-min) (point-max))
+    (message "sorting the buffer...")
+    (skk-sort-lines nil (point-min) (point-max))
+    ;;(sort-fields 1 (point-min) (point-max))
+    (message "sorting the buffer...done")
     (skk-merge-rm-dups donot-countdown)
     (message "񤤤Ƥ롣")
-    (write-region (point-min) (point-max) "NEW-SKK-JISYO" nil 'nomsg)
+    (write-region (point-min) (point-max)
+		  (concat default-directory "NEW-SKK-JISYO") nil 'nomsg)
     (message "Done!")
-    (kill-buffer " *work*")))
+    ;;(kill-buffer " *work*")
+    ))
 
 (defun skk-merge-load (&rest files)
   (switch-to-buffer (get-buffer-create " *work*"))
@@ -350,6 +364,16 @@
    files)
   (j-delete-okuri-entries))
 
+(defun skk-sort-lines  (reverse beg end)
+  "if REVERS sort by the reverse order. sort the region by calling
+emacs-sort."
+  (let ((tmp-file
+	 (expand-file-name (concat default-directory "SKK-TMP-FILE"))))
+    (write-region beg end tmp-file nil 'nomsg)
+    (call-process skk-emacs-sort nil nil nil (if reverse "-r" "-n") tmp-file)
+    (delete-region beg end)
+    (insert-file-contents tmp-file)
+    (call-process "/bin/rm" nil nil nil tmp-file)))
 
 (defun skk-merge-rm-dups (shizukani)
   (let (p q (curl 0) (lines 0))
@@ -416,7 +440,7 @@
 (defun j-vadd (vect1 vect2)
   "add elements of vect2 not in vect1 to vect1."
   (let ((v vect1)			; result vector
-	(i 0)				; roop counter for vect2
+	(i 0)				; loop counter for vect2
 	w				; element in vect2
 	e1				; element in vect1
 	(l1 (length vect1))		; length of vect1
@@ -539,7 +563,7 @@
     (goto-char (point-min))
     (search-forward ";; okuri-nasi")
     (beginning-of-line)
-    (sort-lines t (point-min) (point))))
+    (skk-sort-lines t (point-min) (point))))
 
 ;; Count the number of entries in a jisyo buffer. For instance the following
 ;; line contains 4 entries.
@@ -559,6 +583,14 @@
 (defun skk-latexinfo-format-buffer ()
   (interactive)
   (let (p)
+    ;; remove iftex part
+    (goto-char (point-min))
+    (while (re-search-forward "^\\\\begin{iftex}" nil t)
+      (beginning-of-line)
+      (setq p (point))
+      (re-search-forward "^\\\\end{iftex}")
+      (end-of-line)
+      (delete-region p (1+ (point))))
     (latexinfo-format-buffer t)
     (goto-char (point-min))
     (while (search-forward "[{[" nil t)
